#include "Quadrature.h"
#include "FiniteElemSpace.h"

namespace AMDiS {

  template<typename T>
  T* SubAssembler::getVectorAtQPs(DOFVectorBase<T>* vec, 
				  const ElInfo* elInfo,
				  Quadrature *quad)
  {
    FUNCNAME("SubAssembler::getVectorAtQPs()");

    TEST_EXIT_DBG(vec)("No DOF vector!\n");
    TEST_EXIT_DBG(elInfo->getMesh() == vec->getFeSpace()->getMesh())
      ("Vector and FE space do not fit together!\n");

    Quadrature *localQuad = quad ? quad : quadrature;

    if (valuesAtQPs[vec] && valuesAtQPs[vec]->valid) 
      return &(valuesAtQPs[vec]->values[0]);

    if (!valuesAtQPs[vec])
      valuesAtQPs[vec] = new ValuesAtQPs;
    valuesAtQPs[vec]->values.resize(localQuad->getNumPoints());

    T *values = &(valuesAtQPs[vec]->values[0]);
    bool sameFeSpaces = 
      vec->getFeSpace() == rowFeSpace || vec->getFeSpace() == colFeSpace;

    if (opt && !quad && sameFeSpaces) {
      const BasisFunction *psi = rowFeSpace->getBasisFcts();
      const BasisFunction *phi = colFeSpace->getBasisFcts();
      if (vec->getFeSpace()->getBasisFcts() == psi)
	psiFast = updateFastQuadrature(psiFast, psi, INIT_PHI);
      else if(vec->getFeSpace()->getBasisFcts() == phi)
	phiFast = updateFastQuadrature(phiFast, phi, INIT_PHI);      
    }

    // calculate new values
    const BasisFunction *basFcts = vec->getFeSpace()->getBasisFcts();

    if (opt && !quad && sameFeSpaces) {
      if (psiFast->getBasisFunctions() == basFcts) {
	vec->getVecAtQPs(elInfo, NULL, psiFast, values);
      } else if (phiFast->getBasisFunctions() == basFcts) {
	vec->getVecAtQPs(elInfo, NULL, phiFast, values);
      } else {
	vec->getVecAtQPs(elInfo, localQuad, NULL, values);
      }
    } else {
      vec->getVecAtQPs(elInfo, localQuad, NULL, values);
    }

    valuesAtQPs[vec]->valid = true;

    return values;
  }


  template<typename T>
  T* SubAssembler::getVectorAtQPs(DOFVectorBase<T>* vec, 
				  const ElInfo* smallElInfo,
				  const ElInfo* largeElInfo,
				  Quadrature *quad)
  {
    FUNCNAME("SubAssembler::getVectorAtQPs()");

    TEST_EXIT_DBG(vec)("No DOF vector!\n");

    Quadrature *localQuad = quad ? quad : quadrature;

    if (!valuesAtQPs[vec])
      valuesAtQPs[vec] = new ValuesAtQPs;

    valuesAtQPs[vec]->values.resize(localQuad->getNumPoints());
    T *values = &(valuesAtQPs[vec]->values[0]);
    valuesAtQPs[vec]->valid = true;
    vec->getVecAtQPs(smallElInfo, largeElInfo, localQuad, NULL, values);

    return values;
  }

}
