#include "time/RosenbrockStationary.h"
#include "ProblemVec.h"
#include "SystemVector.h"
#include "OEMSolver.h"

namespace AMDiS {

  void RosenbrockStationary::acceptTimestep()
  {
    *solution = *newUn;
    *unVec = *newUn;
  }

  void RosenbrockStationary::init()
  {
    stageSolution = new SystemVector(*solution);
    unVec = new SystemVector(*solution);
    newUn = new SystemVector(*solution);
    tmp = new SystemVector(*solution);
    lowSol = new SystemVector(*solution);
    
    stageSolutions.resize(rm->getStages());
    for (int i = 0; i < rm->getStages(); i++)
      stageSolutions[i] = new SystemVector(*solution);
    
    phiSum = new DOFVector<double>(feSpaces[0], "phiSum");
    tmpDof = new DOFVector<double>(feSpaces[0], "phiSum");
  }


  void RosenbrockStationary::buildAfterCoarsen(AdaptInfo *adaptInfo, Flag flag,
					       bool asmMatrix, bool asmVector)
  {        
    if (first) {
      first = false;
      *unVec = *solution;
    }
    
    *newUn = *unVec;    
    *lowSol = *unVec;
    
    for (int i = 0; i < rm->getStages(); i++) {      
      *stageSolution = *unVec;
      for (int j = 0; j < i; j++) {
	*tmp = *(stageSolutions[j]);
	*tmp *= rm->getA(i, j);
	*stageSolution += *tmp;
      }
      
      phiSum->set(0.0);
      for (int j = 0; j < i; j++) {
	*tmpDof = *(stageSolutions[j]->getDOFVector(0));
	*tmpDof *= (rm->getC(i, j) / *tauPtr);
	*phiSum += *tmpDof;
      }
      
      ProblemVec::buildAfterCoarsen(adaptInfo, flag, (i == 0), asmVector);
      solver->setMultipleRhs(i != 0);
      ProblemVec::solve(adaptInfo);
      
      *(stageSolutions[i]) = *solution;
      
      *tmp = *solution;
      *tmp *= rm->getM1(i);
      *newUn += *tmp;
      
      *tmp = *solution;
      *tmp *= rm->getM2(i);
      *lowSol += *tmp;
    }
    
    for (int i = 0; i < nComponents; i++) {
      (*(lowSol->getDOFVector(i))) -= (*(newUn->getDOFVector(i)));
      adaptInfo->setTimeEstSum(lowSol->getDOFVector(i)->l2norm(), i);
    }
  }


  void RosenbrockStationary::solve(AdaptInfo *adaptInfo, bool fixedMatrix)
  {}

}
